//--------------------------------------------------------------------------
//
//  loadmap - an IDA plugin, which loads a VC++/BCC map file.
//  Copyright (C) 2000 Toshiyuki Tega. All rights reserved.
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the author be held liable for any damages
//  arising from the use of this software.
//
//--------------------------------------------------------------------------

//  IDA SDK headers.
#include  <ida.hpp>
#include  <idp.hpp>
#include  <loader.hpp>
#include  <segment.hpp>
#include  <bytes.hpp>
#include  <name.hpp>

//  other headers.
#include  <cstring>
#include  "mapread.h"

//--------------------------------------------------------------------------
//  init()
//    Initialize plugin.
//    Though I've tested it only for PE, it may work for other formats.
//--------------------------------------------------------------------------
int init(void)
{
  return (f_PE == inf.filetype) ? PLUGIN_OK : PLUGIN_SKIP;
}

//--------------------------------------------------------------------------
//  run()
//    Main function of plugin. "arg" is not used here.
//--------------------------------------------------------------------------
void run(int /* arg */)
{
  char   map_name[256];
  strcpy(map_name, get_input_file_path());
  strcpy(map_name + strlen(map_name) - 3, "map");

  map_reader  reader(map_name);

  //  First, seek to the beginning of the symbol table.
  if(reader.skip_header())
  {
    const unsigned long max_seg = get_segm_qty();
    unsigned long       seg, addr;
    char *              name;

    //  Repeat until we eat all the lines in the map file.
    while(false == reader.eof())
      if(reader.load_symbol(&seg, &addr, &name) && --seg < max_seg)
      {
        unsigned long la = addr + getnseg(seg)->startEA;
        flags_t       f = getFlags(la);

        //  Add name if there's no meaningful name assigned.
        if(false == has_name(f) || has_dummy_name(f) || has_auto_name(f))
          set_name(la, name, SN_NOWARN);
      }
  }

  msg("Symbols in the map file loaded successfully.\n");
}

//--------------------------------------------------------------------------
//  Plugin information.
//--------------------------------------------------------------------------
char wanted_name[]   = "Import MAP file";
char wanted_hotkey[] = "Alt-M";
char comment[]       = "LOADMAP loads symbols from a VC++/BCC map file.";
char help[]          = "LOADMAP, VC++/BCB map file import plugin\n"
                       "\n"
                       "This module reads an accompanying map file,\n"
                       "and loads symbols into IDA database.\n"
                       "\n"
                       "The map file should be located in the same\n"
                       "directory as the executable.\n";

//--------------------------------------------------------------------------
//  Plugin description block.
//--------------------------------------------------------------------------
extern "C" plugin_t PLUGIN =
{
  IDP_INTERFACE_VERSION,
  0,                    //  Plugin flags (not used).
  init,                 //  Initialize.
  0,                    //  Terminate (not used).
  run,                  //  Main function.
  comment,
  help,
  wanted_name,
  wanted_hotkey
};
