//--------------------------------------------------------------------------
//
//  loadmap - an IDA plugin, which loads a VC++/BCC map file.
//  Copyright (C) 2000 Toshiyuki Tega. All rights reserved.
//
//  This software is provided 'as-is', without any express or implied
//  warranty.  In no event will the author be held liable for any damages
//  arising from the use of this software.
//
//--------------------------------------------------------------------------

#if       !defined(INCLUDED_MAPREAD)
#define            INCLUDED_MAPREAD

#include  <cstdio>

class map_reader
{
  bool get_hex(unsigned long * val, bool init);
  bool get_string(char ** str);
  bool get_line();
  bool get_symbol(unsigned long * seg_no,
                  unsigned long * offset,
                  char **         name);

  enum map_reader_state
  {
    SKIPPING_HEADER = 0,
    LOADING_SYMBOL,
    FINISHED,
  };

  enum map_file_type
  {
    MAP_FILE_TYPE_UNKNOWN = 0,
    MAP_FILE_TYPE_VC,
    MAP_FILE_TYPE_BCC,
  };

  static const int  BUFSIZE = 4096;
  char              m_buf[BUFSIZE];
  int               m_pos;
  std::FILE *       m_fp;
  map_reader_state  m_state;
  map_file_type     m_type;

public:
  map_reader(const char * map_name);
  ~map_reader();

  bool  skip_header();
  bool  load_symbol(unsigned long * seg_no, unsigned long * offset, char ** name);
  bool  eof();
};

#endif // !defined(INCLUDED_MAPREAD)
